
#include <ESP8266WiFi.h>                                                    // esp8266 library
#include <FirebaseArduino.h>                                                // firebase library
#include <TinyGPS++.h>
#include<SoftwareSerial.h>

#define FIREBASE_HOST "PROJECT_NAME"                          // the project name address from firebase id
#define FIREBASE_AUTH "PROJECT_SECRETKEY"            // the secret key generated from firebase

#define WIFI_SSID "WIFI_NAME"                                             // input your home or public wifi name 
#define WIFI_PASSWORD "WIFI_PASSWORD"                                    //password of wifi ssid

TinyGPSPlus gps;                                                        //GPS object
SoftwareSerial ss(4,5);                                                 //Serial pins
void setup() {
  pinMode(D6, OUTPUT);                                                 //LEDs
  pinMode(D7, OUTPUT);
  pinMode(D0, OUTPUT);
  pinMode(D5, INPUT_PULLUP);                                         // Input from push button
  Serial.begin(115200);
  ss.begin(9600);                                                   // start GPS communication
  digitalWrite(D6,HIGH);
  digitalWrite(D7,HIGH);
  delay(1000);  
  digitalWrite(D6,LOW);
  digitalWrite(D7,LOW);
               
  WiFi.begin(WIFI_SSID, WIFI_PASSWORD);                                     //try to connect with wifi
  Serial.print("Connecting to ");
  Serial.print(WIFI_SSID);
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(500);
   }
  Serial.println();
  Serial.print("Connected to ");
  Serial.println(WIFI_SSID);
  Serial.print("IP Address is : ");
  Serial.println(WiFi.localIP());                                            //print local IP address
  Firebase.begin(FIREBASE_HOST, FIREBASE_AUTH);                              // connect to firebase
  digitalWrite(D6, HIGH);
  delay(1000);
  digitalWrite(D6, LOW);
                                                              
}

void loop() { 
  int btncnt = 0;                                                      // count of button press
  Serial.println("Looping");
  Serial.println(digitalRead(D5));
  if(digitalRead(D5) == LOW)
  { 
    btncnt = 1;
    long starttime = millis();
    delay(300);                                                     // giving time for release of button
    while(millis() - starttime <=1000){                             // if not released or pressed once again in 1 sec put into notification mode 
       if(digitalRead(D5) == LOW)
         btncnt = 2;
       yield();                                                     //remember to put yield() in blocking functions otherwise ESP8266 will crash
      } 
    }
  if(btncnt == 2){                                                 // Notification Mode
    Serial.println("Notification");
    digitalWrite(D0, HIGH);
    delay(200);
    digitalWrite(D0, LOW);
    notification();
    }
  else if(btncnt == 1)                                            // Store Mode
   {
    Serial.println("Store");
    digitalWrite(D7, HIGH);
    delay(200);
    digitalWrite(D7, LOW); 
    store();
    }
  delay(100);
}
void store(){
  while(!gps.location.isValid()){                                //Waits till the location is valid
    Serial.println("No Location Data");
    digitalWrite(D7, HIGH);
    smartDelay(500);
    digitalWrite(D7, LOW);
    if(digitalRead(D5) == LOW)
    { 
      delay(1000);                                             //This delay is to let user take hands off the button, so that no false triggers happen
      break;
      }
    yield(); 
   }
  if(gps.location.isValid()){
    int count = Firebase.getInt("count");                                      //Take the current count of data
    if (Firebase.failed()) {
      Serial.print("count failed:");
      Serial.println(Firebase.error());  
      return;
    }
    float lati = gps.location.lat();
    Serial.println(lati);
    float lon  = gps.location.lng();
    Serial.println(lon);
    String tlatchannel = String("/loc/targetloc/lati/")+String(count);                 //setup path and send readings
    String tlonchannel = String("/loc/targetloc/longi/")+String(count);
    Firebase.setFloat(tlatchannel, lati);                                  
    if (Firebase.failed()) {
      Serial.print("tlatchannel failed:");
      Serial.println(Firebase.error());  
      return;
    }
    Firebase.setFloat(tlonchannel, lon);                              
    if (Firebase.failed()) {
      Serial.print("tlonchannel failed:");
      Serial.println(Firebase.error());  
      return;
    }
    digitalWrite(D6, HIGH);
    delay(500);
    digitalWrite(D6, LOW); 
    Serial.println("Location Sent");
    Firebase.setInt("count",count+1);
    smartDelay(500);                                              //a different type of delay so that gps object is constantly updated
    } 
 
  }
void notification(){
    while(!gps.location.isValid()){                              //Waits till the location is valid and breaks if user press the push button
    Serial.println("No Location Data");  
    digitalWrite(D7, HIGH);
    smartDelay(500);
    digitalWrite(D7, LOW);
    if(digitalRead(D5) == LOW)
    { 
      delay(1000);
      break;
      } 
    yield();
   }
   while(gps.location.isValid()){                                         //Since this mode is to work constantly, it is put inside a while loop
    
    float lati = gps.location.lat();
    Serial.println(lati);
    float lon  = gps.location.lng();
    Serial.println(lon);
    digitalWrite(D6, HIGH);
    delay(200);
    digitalWrite(D6, LOW); 
    String clatchannel = String("/loc/currloc/lati");
    String clonchannel = String("/loc/currloc/longi");
    Firebase.setFloat(clatchannel, lati);                                  //setup path and send readings
    if (Firebase.failed()) {
      Serial.print("clatchannel failed:");
      Serial.println(Firebase.error());  
      return;
    }
    Firebase.setFloat(clonchannel, lon);                                //setup path and send readings
    if (Firebase.failed()) {
      Serial.print("clonchannel failed:");
      Serial.println(Firebase.error());  
      return;
    }
    float distance = Firebase.getFloat("distance");
    if (Firebase.failed()) {
      Serial.print("distance failed:");
      Serial.println(Firebase.error());  
      return;
    }
    Serial.println(distance);
    if(distance < 50){                                          // A variable blink time is made to make difference between the distance
      for(int i = 0; i < 500/(50-distance); i++){
      digitalWrite(D0, HIGH);
      delay((50-distance)/2);                                  // If you are closer, it blinks faster and vice versa
      digitalWrite(D0, LOW);
      delay((50-distance)/2);}
    }
    digitalWrite(D6, HIGH);
    delay(500);
    digitalWrite(D6, LOW);
    yield(); 
    smartDelay(500);
    if(digitalRead(D5) == LOW)
    { 
      delay(1000);
      break;
      }
    }
    
  }
static void smartDelay(unsigned long ms)                
  {
    unsigned long start = millis();
    do 
     {
       while (ss.available())
          gps.encode(ss.read());
     } while (millis() - start < ms);
   }
